// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2009 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.java.expression.operator;

import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.PrettyPrinter;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.TypeConverter;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.expression.Operator;
import de.uka.ilkd.key.java.reference.ExecutionContext;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.util.ExtList;

/**
 *  Shift right.
 *  @author <TT>AutoDoc</TT>
 */
public class ShiftRight extends Operator {

    /**
     *      Shift right.
     */
    public ShiftRight() {}

    /**
     *      Shift right.
     *      @param lhs an expression.
     *      @param rhs an expression.
     */
    public ShiftRight(Expression lhs, Expression rhs) {
        super(lhs, rhs);       
    }

    /**
     * Constructor for the transformation of COMPOST ASTs to KeY.
     * The first occurrence of an Expression in the given list is taken as
     * the left hand side 
     * of the expression, the second occurrence is taken as the right hand
     * side of the expression.
     * @param children the children of this AST element as KeY classes.
     */
    public ShiftRight(ExtList children) {
	super(children);
    }


    /**
     *      Get arity.
     *      @return the int value.
     */

    public int getArity() {
        return 2;
    }

    /**
     *      Get precedence.
     *      @return the int value.
     */

    public int getPrecedence() {
        return 4;
    }

    /**
     *      Get notation.
     *      @return the int value.
     */

    public int getNotation() {
        return INFIX;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnShiftRight(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printShiftRight(this);
    }


    public KeYJavaType getKeYJavaType(Services javaServ, ExecutionContext ec) {
	final TypeConverter tc = javaServ.getTypeConverter();
	return tc.getPromotedType
	    (tc.getKeYJavaType((Expression)getChildAt(0), ec));
    }
}
